#!/usr/bin/env python3
'''
Configuration.
'''

import logging
import pathlib

import yaml

from jnp.common import NAME
from jnp.text import docstring_to_comment


LOGGER = logging.getLogger(__name__)


def _parse_variables(variables, defaults=None):
    '''
    Parse the CSS variables from the configurationf file and return a generator
    over key-value pairs. This is used to enable nesting of related variables in
    dicts. For example, instead of specifying each background property by it's
    full name (background-color, background-image, etc.), the properties can be
    nested under a common "background" field (background: color, image, etc.).

    Args:
        variables:
            The dict of CSS variables with possibly nested dicts.

        defaults:
            Default values for the dict to fill in missing values, including
            those from nested dicts.

    Returns:
        A generator over key-value pairs with the expanded variable names.
    '''
    all_keys = set((*variables.keys(), *defaults.keys()))
    for key in sorted(all_keys):
        value = variables.get(key, defaults.get(key))
        if isinstance(value, dict):
            subdefaults = defaults.get(key, {})
            for subkey, value in _parse_variables(value, defaults=subdefaults):
                yield f'{key}-{subkey}', value
        yield key, value


class Config():
    '''
    Configuration file reader.
    '''
    CSS_VARIABLES = {
        'jnp-main-color': '#1488ca',
        'jnp-h1-color': '#ffcd1c',
        'jnp-background': {
            'attachment': 'fixed',
            'color': '#ffffff',
            'image': 'none',
            'position': 'center',
            'repeat': 'no-repeat',
            'size': '100% 100%',
        },
        'jnp-radius': '10pt',
    }

    def __init__(self, path=None):
        '''
        Args:
            path:
                The path to the configuration file. If None, defaults values
                will be used for all settings relative to the current working
                directory.
        '''
        if path is not None:
            path = pathlib.Path(path).resolve()
        self.path = path
        self._config = None
        self._cwd = pathlib.Path.cwd()

    @property
    def _root_dir(self):
        '''
        The root directory relative to which paths should be interpretted.
        '''
        if self.path:
            return self.path.parent
        return self._cwd

    @property
    def config(self):
        '''
        The configuration file object, or None if no path was given.
        '''
        if self.path and not self._config:
            LOGGER.info('Loading %s', self.path)
            with self.path.open('rb') as handle:
                self._config = yaml.safe_load(handle)
        return self._config

    def _get(self, key, default=None):
        '''
        Try to get a key from the configuration file.

        Args:
            key:
                The key to get.

            default:
                The default value to use when the key is not found.

        Returns:
            The value, or the default if the key was not found.
        '''
        try:
            value = self.config[key]
        except (TypeError, KeyError):
            value = None
        if value is None:
            value = default
        return value

    def _get_example_entry(self, name, *default):
        '''
        Get an entry for the example file.

        Args:
            name:
                The field/attribute name.

            default:
                If given, it will be used in place of the class's default value
                for this attribute.

        Returns:
            A YAML stub.
        '''
        doc = getattr(self.__class__, name).__doc__
        doc = docstring_to_comment(doc)
        if default:
            default = default[0]
        else:
            default = getattr(self, name)
            if isinstance(default, pathlib.Path):
                if default.is_relative_to(self._root_dir):
                    default = default.relative_to(self._root_dir)
                default = str(default)
        line = yaml.dump({name: default})
        return f'{doc}\n{line}'

    @property
    def example(self):
        '''
        An example configuration file.
        '''
        return f'''# jnp configuration file

{self._get_example_entry('transition')}
{self._get_example_entry('aspect_ratio')}
{self._get_example_entry('width')}
{self._get_example_entry('public_dir')}
{self._get_example_entry('input_dir')}
{self._get_example_entry('stylesheets')}
{self._get_example_entry('css_variables', self.CSS_VARIABLES)}
{self._get_example_entry('index_template', None)}
{self._get_example_entry('placeholder')}
'''.strip()

    @property
    def transition(self):
        '''
        The reveal.js slide transition effect. This is passed through to
        nbconvert's SlidesExporter.reveal_transition option. For the list of
        supported values, see

            https://nbconvert.readthedocs.io/en/latest/config_options.html
        '''
        return self._get('transition', default='slide')

    @property
    def aspect_ratio(self):
        '''
        The presentation's aspect ratio.
        '''
        return self._get('aspect_ratio', default=1.6)

    @property
    def width(self):
        '''
        The presentation's width. The presentation will scale to the screen so
        this basically just adjusts the size of text and other elements.
        '''
        return self._get('width', default=1100)

    @property
    def public_dir(self):
        '''
        The directory in which to publish the presentations.
        '''
        return self._root_dir / self._get('public_dir', default='public')

    @property
    def stylesheets(self):
        '''
        CSS stylesheets to include in the exported HTML slides. These will be
        interpretted relative to the public directory.

        The default list includes the custom stylesheets provided by jnp. These
        can be supplemented, modified or replaced with the users own
        stylesheets. The default files will be created automatically when jnp
        exports the slides.

        For example, "../css/custom.css" will load "<public_dir>/css/custom.css"
        from a notebook located in "<public_dir>/notebooks".
        '''
        subpath = f'../css/{NAME}/'
        names = ('main.css', 'sizes.css', 'reveal.css')
        return self._get('stylesheets', default=[f'{subpath}{name}' for name in names])

    @property
    def css_variables(self):
        '''
        The default main.css stylesheet supports limited configuration via the
        following variables:

            jnp-main-color:
                The main color of the presentation's theme (H1 header
                backgrounds, H2 header text, links, etc.)

            jnp-h1-color:
                The text color of H1 headers displayed on jnp_main_color
                backgrounds.

            jnp-background:
                The CSS background properties (color, image, position, etc.):

                For details, see https://www.w3schools.com/cssref/css3_pr_background.php

            jnp-radius:
                Border radius of H1 header backgrounds, code blocks, images, etc.
        '''
        variables = self._get('css_variables', default=self.CSS_VARIABLES)
        return dict(_parse_variables(variables, defaults=self.CSS_VARIABLES))

    @property
    def _resource_dir(self):
        '''
        An iterable of this package's resource file paths.
        '''
        #  m_path = importlib.resources.files('jnp.resources')
        #  return pathlib.Path(m_path.joinpath(''))
        return pathlib.Path(__file__).resolve().parent / 'resources'

    @property
    def static_css_files(self):
        '''
        An iterable of the static CSS filepaths.
        '''
        for path in self._resource_dir.glob('*.css'):
            yield path

    @property
    def input_dir(self):
        '''
        The directory containing the Jupyter notebooks and associated files for
        the presentation. This directory should contain a subdirectory named
        "notebooks" with the Jupyter notebook files (.ipynb file extension). The
        rest of the files in the directory will be symlinked to the output
        directory when jnp is run.
        '''
        return self._root_dir / self._get('input_dir', default='presentations')

    @property
    def notebook_dir(self):
        '''
        The directory containing the Jupyter notebook files.
        '''
        return self.input_dir / 'notebooks'

    @property
    def notebooks(self):
        '''
        An iterable over Jupyter notebook files to convert to presentations.
        '''
        yield from self.notebook_dir.glob('*.ipynb')

    @property
    def index_template(self):
        '''
        The index page template file path.
        '''
        path = self._get('index_template')
        if path:
            return self.path.parent / path
        return self._resource_dir / 'index.html'

    @property
    def placeholder(self):
        '''
        The placeholder variable for the list of links in the index template.
        '''
        return self._get('placeholder', default='%NOTEBOOK_LINKS%')



#!/usr/bin/env python3
'''
Jupyter notebook functions.
'''

import logging
import pathlib
import re
import subprocess
import tempfile
from html.parser import HTMLParser
from xml.sax.saxutils import escape as xml_escape

from IPython.display import Code

from jnp.common import ENCODING
from jnp.file import copy


LOGGER = logging.getLogger(__name__)


class LinkParser(HTMLParser):
    '''
    HTML parser to check if a link's text matches its "href" attribute and
    append the "raw_link" class if it does.
    '''
    def __init__(self):
        super().__init__()
        self.attrs = {}
        self.data = None

    def handle_starttag(self, tag, attrs):
        self.attrs = dict(attrs)

    def handle_data(self, data):
        self.data = data

    def get_replacement(self):
        '''
        Return replacement text for the link if the "href" is the same as the
        link's text, else None.
        '''
        href = self.attrs.get('href', '')
        if not href or href in self.data:
            classes = self.attrs.get('class', set())
            if classes:
                classes = set(classes.split(','))
            classes.add('explicit_url')
            self.attrs['class'] = ','.join(sorted(classes))
            attrs = ' '.join(
                f'{key}="{xml_escape(value)}"'
                for (key, value) in sorted(self.attrs.items())
            )
            return f'<a {attrs}>{self.data}</a>'
        return None


def _add_link_classes(match):
    '''
    Add a custom CSS class to disable the display of the link's "href" URL after
    the link's text if the text already contains the URL. This function is
    passed to re.sub to update all of the links in the HTMl document.

    Args:
        match:
            The re.Match object.

    Returns:
        The update link element as a string.
    '''
    link = match.group(0)
    parser = LinkParser()
    parser.feed(link)
    replacement = parser.get_replacement()
    if replacement:
        return replacement
    return link


def _get_stylesheet_links(stylesheets):
    '''
    Get HTML link elements for the given stylesheet URLs.

    Args:
        stylesheets:
            An iterable of stylesheet URLs.

    Returns:
        A generator over HTML link elements.
    '''
    for stylesheet in stylesheets:
        yield f'<link rel="stylesheet" href="{xml_escape(stylesheet)}" />'


def notebook_html_cell(config):
    '''
    Get a Jupyter notebook HTML cell for including the stylesheets directly in
    the notebook.

    Args:
        config:
            The configuration object.

    Returns:
        A string with the cell contents.
    '''
    links = '\n'.join(_get_stylesheet_links(config.stylesheets))
    if links:
        return f'%%html\n{links}'
    return '# The configuration file does not specify any stylesheets.'


def nbconvert(config, path, out_dir):
    '''
    Convert a notebook to slides. This inserts some custom code into the output
    to handle sizing and aspect ratios.

    Args:
        config:
            The configuration object.

        path:
            The path to the notebook.

        out_dir:
            The output directory.

    '''

    # https://nbconvert.readthedocs.io/en/latest/config_options.html
    # https://revealjs.com/themes/
    path = pathlib.Path(path).resolve()
    out_dir = pathlib.Path(out_dir).resolve()
    # Output to a temporary directory (usually tmpfs) to avoid unnecessary disk
    # IO.
    with tempfile.TemporaryDirectory() as tmp_dir:
        tmp_dir = pathlib.Path(tmp_dir)
        cmd = [
            'jupyter', 'nbconvert',
            '--log-level=WARN',
            f'--output-dir={tmp_dir}',
            '--to', 'slides',
            '--SlidesExporter.reveal_theme=white',
            f'--SlidesExporter.reveal_transition={config.transition}',
            '--SlidesExporter.reveal_number=h.v',
            '--TagRemovePreprocessor.remove_input_tags', 'hide_input',
            '--TagRemovePreprocessor.remove_all_outputs_tags', 'hide_output',
            str(path)
        ]
        LOGGER.info('Converting %s', path)
        LOGGER.debug('Running command: %s', cmd)
        subprocess.run(cmd, check=True)

        out_path = (out_dir / path.name).with_suffix('.slides.html')
        tmp_path = tmp_dir / out_path.name
        lines = tmp_path.read_text(encoding=ENCODING).splitlines()
        code = f'''
        const aspect_ratio = {config.aspect_ratio};
        const width = {config.width};
        Reveal.configure({{
          progress: true,
          hash: true,
          width: width,
          height: width / aspect_ratio
        }});
'''
        lines.insert(-6, code)

        for i, link in enumerate(_get_stylesheet_links(config.stylesheets)):
            lines.insert(4 + i, link)

        html = '\n'.join(lines)
        html = re.sub(r'<a .*?</a>', _add_link_classes, html)
        tmp_path.write_text(html)
        copy(tmp_path, out_path)

    return out_path


def display_command_output(cmd, language=None):
    '''
    Display command output in a Jupyter notebook cell.

    Args:
        cmd:
            The cmd, as a list. It will be run with subprocess.run.

        language:
            The language to use for syntax highlighting.

    Returns:
        The
    '''
    data = subprocess.run(cmd, check=True, stdout=subprocess.PIPE).stdout.decode()
    return Code(data=data, language=language)



#!/usr/bin/env python3
'''
Generate CSS files.
'''

import logging
import pathlib

from jnp.common import ENCODING


LOGGER = logging.getLogger(__name__)


def generate_sizes_css(path):
    '''
    Generate a CSS file with a series of incremental classes.
    '''
    path = pathlib.Path(path).resolve()
    LOGGER.debug('Creating %s', path)
    path.parent.mkdir(parents=True, exist_ok=True)
    with path.open('w', encoding=ENCODING) as handle:
        # Jupyter notebook strips out inline styles such as flex-basis that can
        # change the page layout so we create custom classes with 5% increments
        # as a work-around.
        for pct in range(5, 100, 5):
            handle.write(f'''.fb{pct} {{
  flex-basis: {pct}%;
}}
''')
        # reveal.js code height classes.
        for height in range(5, 40, 1):
            handle.write(f'''.reveal .ch{height} pre,
.reveal .ch{height} code,
.reveal .ch{height} pre code {{
  max-height: calc({height} * var(--jp-code-font-size)) !important;
  overflow: auto;
}}
''')



#!/usr/bin/env python3
'''
Presentation export manager.
'''

import logging
import os
import pathlib
import shutil
import tempfile

from jnp.common import ENCODING, NAME
from jnp.config import Config
from jnp.css import generate_sizes_css
from jnp.file import copy
from jnp.notebook import nbconvert


LOGGER = logging.getLogger(__name__)


class Manager():
    '''
    Presentation export manager.
    '''
    def __init__(self, config_path=None):
        '''
        Args:
            config_path:
                The path to the configuration file. If None, it will use an
                internal configuration file with default options.
        '''
        self.config = Config(path=config_path)

    @property
    def public_dir(self):
        '''
        The directory in which to publish the presentations and associated files.
        '''
        return self.config.public_dir

    @property
    def public_notesbooks_dir(self):
        '''
        The directory of published presentations.
        '''
        return self.public_dir / 'notebooks'

    def clean(self):
        '''
        Remove the public directory.
        '''
        pub_dir = self.public_dir
        LOGGER.info('Removing %s', pub_dir)
        try:
            shutil.rmtree(pub_dir)
        except FileNotFoundError:
            pass

    @staticmethod
    def _insert_links(line, placeholder, links):
        '''
        '''
        # Preserve indentation.
        start, end = line.split(placeholder, 1)
        link_indent = ' ' * (len(start) - len(start.lstrip()))
        end_indent = link_indent
        if start.strip():
            yield start
            link_indent += ' ' * 2
        for link in links:
            yield f'{link_indent}{link}'
        if end.strip():
            yield f'{end_indent}{end}'

    @staticmethod
    def _get_index_display_name(path):
        '''
        Get the display name of the given path for the index page.

        Args:
            path:
                An instance of pathlib.Path.

        Returns:
            The display name.
        '''
        name = path.name.split('.')
        for suffix in ('html', 'slides'):
            if name[-1] == suffix:
                name = name[:-1]
        return '.'.join(name)

    def _publish_index_page(self):
        '''
        Publish the index page with links to the published notebooks.
        '''
        config = self.config
        pub_dir = self.public_dir
        pub_notebook_dir = self.public_notesbooks_dir
        hrefs = (
            path.relative_to(pub_dir)
            for path in pub_notebook_dir.glob('*.html')
        )
        links = sorted(
            f'<li><a href="{href}">{self._get_index_display_name(href)}</a></li>'
            for href in hrefs
        )
        index_path = pub_dir / 'index.html'
        placeholder = config.placeholder
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_dir = pathlib.Path(tmp_dir)
            tmp_path = tmp_dir / index_path.name
            with tmp_path.open('w', encoding=ENCODING) as handle:
                for line in config.index_template.read_text(encoding=ENCODING).splitlines():
                    if placeholder in line:
                        for mod_line in self._insert_links(line, placeholder, links):
                            handle.write(f'{mod_line}\n')
                    else:
                        handle.write(f'{line}\n')
            copy(tmp_path, index_path)

    def _interpolate_css_variables(self, inpath, outpath):
        ''''
        Interpolate the CSS variables into the input path and save it to the
        output path.

        Args:
            inpath:
                The input path.

            outpath:
                The output path.
        '''
        variables = Config.CSS_VARIABLES.copy()
        variables.update(self.config.css_variables)
        variable_block = '\n'.join(
            f'  --{name}: {value};'
            for name, value in variables.items()
        )
        css = inpath.read_text(encoding=ENCODING).replace(
            r'/* %VARIABLES% */',
            variable_block
        )
        outpath.parent.mkdir(parents=True, exist_ok=True)
        outpath.write_text(css, encoding=ENCODING)

    def _copy_input_files(self):
        '''
        Copy input files to the ouput directory.
        '''
        # Switch to pathlib.walk when Python 3.12 becomes widespread.
        for root, dirs, fils in os.walk(self.config.input_dir):
            root = pathlib.Path(root)
            dirs[:] = [d for d in dirs if root / d != self.config.notebook_dir]
            for fil in fils:
                inpath = root / fil
                rel_path = inpath.relative_to(self.config.input_dir)
                outpath = self.config.public_dir / rel_path
                copy(inpath, outpath)

    def copy_static_files(self, dir_path):
        '''
        Copy jnp's internal static files to a directory.

        Args:
            dir_path:
                The output directory.
        '''
        css_dir = dir_path / f'css/{NAME}'
        for path in self.config.static_css_files:
            out_path = css_dir / path.name
            if path.name == 'main.css':
                self._interpolate_css_variables(path, out_path)
            else:
                copy(path, out_path)
        generate_sizes_css(css_dir / 'sizes.css')

    def publish(self):
        '''
        Publish all files to the public directory.
        '''
        config = self.config

        self.copy_static_files(self.public_dir)
        self._copy_input_files()

        pub_notebook_dir = self.public_notesbooks_dir
        for path in config.notebooks:
            nbconvert(config, path, pub_notebook_dir)

        self._publish_index_page()



#!/usr/bin/env python3
'''
Fix for syntax highlighting with Code in Jupyter Lab:
https://github.com/ipython/ipython/issues/11747
'''
from IPython.display import Code
from pygments import highlight
from pygments.formatters import HtmlFormatter


def _jupyterlab_repr_html_(self):
    fmt = HtmlFormatter()
    style = (
        f'<style>{fmt.get_style_defs(".output_html")}\n'
        f'{fmt.get_style_defs(".jp-RenderedHTML")}</style>'
    )
    return style + highlight(self.data, self._get_lexer(), fmt)


Code._repr_html_ = _jupyterlab_repr_html_



#!/usr/bin/env python3
'''
File functions.
'''

import errno
import filecmp
import logging
import os
import pathlib
import shutil


LOGGER = logging.getLogger(__name__)


def symlink(path, target):
    '''
    Create a symlink.

    Args:
        path:
            The symlink path.

        target:
            The symlink target.
    '''
    path = pathlib.Path(path).resolve()
    target = str(target)
    if path == pathlib.Path(target).resolve():
        LOGGER.error('Cannot symlink a path to itself: %s', path)
        return
    path.parent.mkdir(parents=True, exist_ok=True)
    try:
        cur_target = path.readlink()
    except FileNotFoundError:
        pass
    except OSError as err:
        # Not a symlink. Remove it.
        if err.errno == errno.EINVAL:
            if path.is_dir():
                LOGGER.info('Removing directory to create symlink: %s', path)
                shutil.rmtree(path)
            else:
                LOGGER.info('Removing file to create symlink: %s', path)
                path.unlink()
    else:
        if cur_target == target:
            return
        LOGGER.info('Removing old symlink: %s', path)
        path.unlink()
    path.symlink_to(target)


def _copy_dir(src, dst):
    '''
    Copy a directory. Destination files without corresponding source files are
    removed and then each file in the source is copied as necessary to the
    destination.

    Args:
        src:
            The source directory path.

        dst:
            The destination directory path.
    '''
    for root, dirs, fils in os.walk(dst):
        root = pathlib.Path(root)
        rel_root = root.relative_to(dst)
        src_root = src / rel_root

        # Filter and remove directories with no corresponding source.
        missing_dirs = [
            dname for dname in dirs
            if (src_root / dname).is_dir()
        ]
        for dname in missing_dirs:
            shutil.rmtree(root / dname)
        dirs[:] = [d for d in dirs if d not in missing_dirs]

        # Remove files with no corresponding source.
        for fname in fils:
            if not (src_root / fname).is_file():
                (root / fname).unlink()

    for root, _dirs, fils in os.walk(src):
        root = pathlib.Path(root)
        rel_root = root.relative_to(src)
        dst_root = dst / rel_root

        if fils:
            dst_root.mkdir(parents=True, exist_ok=True)

        for fname in fils:
            src_path = root / fname
            dst_path = dst_root / fname
            try:
                if filecmp.cmp(src_path, dst_path, shallow=False):
                    continue
                dst_path.unlink()
            except FileNotFoundError:
                pass
            shutil.copy2(src_path, dst_path)


def copy(src, dst):
    '''
    Copy the source to the destination. This compares existing files to avoid
    redundant copies.

    Args:
        src:
            The source path (either a file or directory).

        dst:
            The destination path.
    '''
    src = pathlib.Path(src).resolve()
    dst = pathlib.Path(dst).resolve()
    if not src.exists():
        LOGGER.warning('%s does not exist.', src)
        return
    if src == dst:
        LOGGER.warning('Cannot copy a path to itself: %s')
        return
    dst.parent.mkdir(parents=True, exist_ok=True)
    if dst.is_symlink():
        dst.unlink()

    if src.is_dir():
        if dst.is_dir():
            _copy_dir(src, dst)
            return
        try:
            dst.unlink()
        except FileNotFoundError:
            pass
        shutil.copytree(src, dst)
    else:
        try:
            if dst.is_dir():
                shutil.rmtree(dst)
            elif filecmp.cmp(src, dst, shallow=False):
                return
            else:
                dst.unlink()
        except FileNotFoundError:
            pass
        shutil.copy2(src, dst)



#!/usr/bin/env python3
'''
Update command output in the README file.

Dependencies;
* [tree](https://gitlab.com/OldManProgrammer/unix-tree)
* [vimdiff](https://www.vim.org/)
'''

import logging
import pathlib
import re
import subprocess
import tempfile


ENCODING = 'utf-8'
LOGGER = logging.getLogger(__name__)
JNP_DIR = pathlib.Path(__file__).resolve().parent.parent
README_PATH = JNP_DIR / 'README.md'


def update_command_output(match):
    '''
    Update the command output for the given regex match.

    Args:
        match:
            The Match object.

    Returns:
        The update output block.
    '''
    indent = match['indent']
    label = match['label'].strip()
    lang = match['lang'].strip()
    tree_cmd = ('tree', '--prune', '--noreport', '-n')
    cmds = {
        'input_dir': [*tree_cmd, 'presentations'],
        'public_dir': [*tree_cmd, 'public'],
        'config_file': ['jnp', '--create-config', '-'],
        'help': ['jnp', '--help'],
        'html_cell': ['jnp', '--html-cell'],
        'index_page': ['jnp', '--copy-index-template', '-']
    }
    try:
        cmd = cmds[label]
    except KeyError:
        LOGGER.error('Unrecognized label in README: %s', label)
        return match.group(0)

    cwd = str(JNP_DIR)
    output = subprocess.run(
        cmd,
        check=True,
        stdout=subprocess.PIPE,
        cwd=cwd,
    ).stdout.decode(ENCODING).strip()
    output = '\n'.join(f'{indent}{line}' for line in output.split('\n'))
    return f'{indent}[output: {label}]: #\n{indent}~~~{lang}\n{output}\n{indent}~~~\n'


def main():
    '''
    Update command output in the README file.
    '''
    text = README_PATH.read_text(encoding=ENCODING)
    regex = re.compile(
        r'^(?P<indent>\s*)\[output: (?P<label>.+?)\]: #\n'
        r'(?P=indent)~~~(?P<lang>.*?)\n'
        r'.*?^(?P=indent)~~~\n',
        re.MULTILINE | re.DOTALL
    )
    text = regex.sub(update_command_output, text)
    with tempfile.TemporaryDirectory() as tmp_dir:
        tmp_dir = pathlib.Path(tmp_dir)
        tmp_path = tmp_dir / README_PATH.name
        tmp_path.write_text(text, encoding=ENCODING)
        subprocess.run(['vimdiff', str(tmp_path), str(README_PATH)], check=True)


if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        pass



#!/usr/bin/env python3
'''
Format text. This is only used for pretty-printing docstrings as comments in the
YAML configuration file output.
'''

import re
import textwrap


def docstring_to_comment(doc):
    '''
    Format a Python docstring as a YAML comment.

    Args:
        doc:
            The docstring.

    Returns:
        The comment string.
    '''
    # Strip superfluous whitespace.
    doc = textwrap.dedent(doc).strip()
    while '\n\n\n' in doc:
        doc = doc.replace('\n\n\n', '\n\n')

    # Split into blocks and then into subblocks based on indentation level.
    blocks = doc.split('\n\n')
    indent_re = re.compile(r'^(\s*)(.*)$')
    for i, block in enumerate(blocks):
        subblocks = []
        for line in block.splitlines():
            match = indent_re.match(line)
            indent = match.group(1)
            content = match.group(2)
            if not subblocks or subblocks[-1]['indent'] != indent:
                subblocks.append({'indent': indent, 'lines': [content]})
            else:
                subblocks[-1]['lines'].append(content)

        for j, subblock in enumerate(subblocks):
            indent = subblock['indent']
            text = '\n'.join(subblock['lines'])
            lines = textwrap.wrap(
                text,
                break_long_words=False,
                initial_indent=indent,
                subsequent_indent=indent
            )
            subblocks[j] = '\n'.join(f'# {line}' for line in lines)

        blocks[i] = '\n'.join(subblocks)

    return '\n#\n'.join(blocks)



#!/usr/bin/env python3
'''
Package entry points.
'''

import argparse
import logging
import pathlib


from jnp.common import ENCODING
from jnp.file import copy
from jnp.manager import Manager
from jnp.notebook import notebook_html_cell


LOGGER = logging.getLogger(__name__)


def _copy(arg_in, arg_out):
    '''
    Convenience function for copying input data to either a file path or STDOUT.

    Args:
        arg_in:
            Either a string or a Path object.

        arg_out:
            Either "-" or a file path.
    '''
    if isinstance(arg_in, pathlib.Path):
        if arg_out == '-':
            print(arg_in.read_text(encoding=ENCODING))
        else:
            copy(arg_in, arg_out)

    else:
        if arg_out == '-':
            print(arg_in)
        else:
            path = pathlib.Path(arg_out).resolve()
            path.parent.mkdir(parents=True, exist_ok=True)
            path.write_text(arg_in, encoding=ENCODING)


def main(args=None):
    '''
    Export Jupyter notebook presentations to a directory.
    '''
    logging.basicConfig(
        style='{',
        format='[{asctime}] {levelname} {message}',
        level=logging.INFO
    )
    parser = argparse.ArgumentParser(description=main.__doc__)
    parser.add_argument(
        '-c', '--config',
        type=pathlib.Path,
        help=('''
            The path to the YAML configuration file.
            If not given, default paths will be used.
        ''')
    )
    parser.add_argument(
        '--create-config',
        metavar='PATH',
        help=('''
            Create a default confuration file at the given path.
            If the path is "-" then print to STDOUT.
        ''')
    )
    parser.add_argument(
        '--copy-static',
        type=pathlib.Path,
        metavar='PATH',
        help=('''
            Copy internal static files to the given path.
            Use this to create modified copies of the internal files.
        ''')
    )
    parser.add_argument(
        '--copy-index-template',
        metavar='PATH',
        help=('''
            Copy the internal index template.
            The template can be used as a starting point for creating a custom
            index page.
            If the path is "-" then print to STDOUT.
        ''')
    )
    parser.add_argument(
        '--html-cell',
        action='store_true',
        help=('''
            Print a Jupyter notebook HTML cell that can be used to display CSS
            styles while editing the notebook. Copy and poste the content to a
            separate "code" cell and execute it.

            Copy the static files to the input directory with --copy-static so
            that Jupyter Lab can find them.
        ''')
    )
    pargs = parser.parse_args(args=args)

    if pargs.create_config:
        _copy(Manager().config.example, pargs.create_config)
        return

    if pargs.copy_index_template:
        _copy(Manager().config.index_template, pargs.copy_index_template)
        return

    manager = Manager(config_path=pargs.config)

    if pargs.html_cell:
        print(notebook_html_cell(manager.config))
        return

    if pargs.copy_static:
        manager.copy_static_files(pargs.copy_static)
        return

    manager.publish()
